from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session

from app import crud, models, schemas
from app.api import deps

router = APIRouter()


@router.get("", response_model=List[schemas.Tag])
def read_tags(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve tags.
    """
    tags = crud.tag.get_multi(db=db, skip=skip, limit=limit)
    return tags


@router.post("", response_model=schemas.Tag)
def create_tag(
    *,
    db: Session = Depends(deps.get_db),
    tag_in: schemas.TagCreate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Create new tag.
    """
    tag = crud.tag.get_by_name(db=db, name=tag_in.name)
    if tag:
        raise HTTPException(
            status_code=400,
            detail="The tag with this name already exists in the system.",
        )
    tag = crud.tag.create(db=db, obj_in=tag_in)
    return tag


@router.get("/{tag_id}", response_model=schemas.Tag)
def read_tag(
    *,
    db: Session = Depends(deps.get_db),
    tag_id: str,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get tag by ID.
    """
    tag = crud.tag.get(db=db, id=tag_id)
    if not tag:
        raise HTTPException(status_code=404, detail="Tag not found")
    return tag


@router.put("/{tag_id}", response_model=schemas.Tag)
def update_tag(
    *,
    db: Session = Depends(deps.get_db),
    tag_id: str,
    tag_in: schemas.TagUpdate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Update a tag.
    """
    tag = crud.tag.get(db=db, id=tag_id)
    if not tag:
        raise HTTPException(status_code=404, detail="Tag not found")
    
    # Check if updating to a name that already exists
    if tag_in.name and tag_in.name != tag.name:
        existing_tag = crud.tag.get_by_name(db=db, name=tag_in.name)
        if existing_tag:
            raise HTTPException(
                status_code=400,
                detail="The tag with this name already exists in the system.",
            )
    
    tag = crud.tag.update(db=db, db_obj=tag, obj_in=tag_in)
    return tag


@router.delete("/{tag_id}", status_code=204, response_model=None)
def delete_tag(
    *,
    db: Session = Depends(deps.get_db),
    tag_id: str,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Delete a tag.
    """
    tag = crud.tag.get(db=db, id=tag_id)
    if not tag:
        raise HTTPException(status_code=404, detail="Tag not found")
    
    crud.tag.remove(db=db, id=tag_id)
    return None